(*  Dialog Zahleneingabe (Integer)
    - Inkrementwerte einstellbar
    - Min- und Maxwerte einstellbar
    - Dezimal- oder Hexanzeige
    - opt. Feld fr Einheiten

    J. Rathlev, IEAP, Uni-Kiel, (rathlev(a)physik.uni-kiel.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Sep. 2002
   mit NumUpDown : Mrz. 2006
   *)

unit InpNumber;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, NumberEd, Spin, JrUtils, ComCtrls;

type
  TInputNumberDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Descriptor: TLabel;
    Units: TLabel;
    NumUpDown: TNumUpDown;
    NumFeld: TEdit;
    procedure FormCreate(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
private
    { Private declarations }
    FIncr : integer;
  public
    { Public declarations }
  end;

(* Zahl eingeben, Ergebnis: "true" bei "ok" *)
function InputInteger(Pos        : TPoint;
                      BottomPos  : boolean;
                      Titel,Desc,
                      AUnits,
                      ACancel    : string;
                      AIncr,
                      ADigits,
                      AMin,AMax  : longint;
                      AHex       : boolean;
                      var Val    : longint) : boolean;

var
  InputNumberDialog: TInputNumberDialog;

implementation

{$R *.DFM}

uses GnuGetText;

{ ------------------------------------------------------------------- }
procedure TInputNumberDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TInputNumberDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
var
  s : string;
  n : integer;

  function Valid (Value : integer) : boolean;
  begin
    with NumUpDown do Result:=(Min<=Value) and (Value<=Max);
    end;

begin
  CanClose:=true;
  if (ModalResult=mrOK) then begin
    try
      n:=StrToInt(NumFeld.Text);
      NumUpDown.Position:=n;
      if not Valid(n) then begin
        with NumUpDown do s:=NumToStr(Min)+' .. '+NumToStr(Max)+')';
        s:=Format(dgettext('dialogs','Error: Out of Range (%s)'),[s]);
        with NumUpDown do begin
          if n<Min then Position:=Min;
          if n>Max then Position:=Max;
          end;
        CanClose:=false;
        end;
    except
      s:=dgettext('dialogs','Error: Invalid input');
      CanClose:=false;
      end;
    if not CanClose then with NumFeld do begin
      with ClientOrigin do ErrorDialog(Caption,s,Point(X+Width,Y+Height));
      SetFocus;
      end;
    end;
  end;

{ ------------------------------------------------------------------- }
(* Zahl eingeben, Ergebnis: "true" bei "ok" *)
function InputInteger(Pos        : TPoint;
                      BottomPos  : boolean;
                      Titel,Desc,
                      AUnits,
                      ACancel    : string;
                      AIncr,
                      ADigits,
                      AMin,AMax  : integer;
                      AHex       : boolean;
                      var Val    : longint) : boolean;
begin
  if not assigned(InputNumberDialog) then
    InputNumberDialog:=TInputNumberDialog.Create(Application);
  with InputNumberDialog do begin
    if Pos.X<0 then Position:=poScreenCenter
    else with Pos do begin
      Position:=poDesigned;
      if X<0 then X:=Left;
      if Y<0 then Y:=Top
      else if BottomPos then y:=y-Height;
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=Pos.x; Top:=Pos.y;
      end;
    Caption:=Titel;

    if length(ACancel)>0 then CancelBtn.Caption:=ACancel;
    Descriptor.Caption:=Desc;
    NumFeld.Width:=Canvas.TextWidth(FillStr('0',ADigits+2));
    with Units do begin
      Left:=NumFeld.Left+NumFeld.Width+20;
      Caption:=AUnits;
      end;
    FIncr:=AIncr;
    with NumUpDown do begin
      Digits:=ADigits; Hex:=AHex;
      Min:=AMin; Max:=AMax;
      Position:=Val;
      end;
    ActiveControl:=NumFeld;
    if ShowModal=mrOK then begin
      Val:=NumUpDown.Position;
      Result:=true;
      end
    else Result:=false;
    Release;
    end;
  InputNumberDialog:=nil;
  end;

end.
